/* id3attr (c) 2000-2003 pinc Software, Axel Drfler (axeld@pinc-software.de)
 *
 * based on id3Attr (c) 1999 Ari Pernick (pernick@uiuc.edu) which is:
 * Based very partially on ID3ren (by Robert Alto (badcrc@tscnet.com)
 * and its BeOS port by Shayne White (shayne@curvedspace.org))
 * Which was based on (c) 1999 Pieter Panman (ppanman@dds.nl)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */
#ifndef ID3ATTR_H
#define ID3ATTR_H

#include <SupportDefs.h>

/************************* ID3 v1 *********************/

struct id3struct {
	char tag[4];
	char title[31];
	char artist[31];
	char album[31];
	char year[5];
	char comment[31];
	int genre;
};


/************************* ID3 v2 *********************/

struct id3v2_header {
	char	identifier[3];	// "ID3"
	uint8	major;			// version
	uint8	minor;
	uint8	flags;
	uint8	size[4];		// size encoded in 7bits

	bool IsValid() const { return !strncmp(identifier, "ID3", 3); }
	uint32 Size() const { return (size[0] << 21UL) | (size[1] << 14UL) | (size[2] << 7UL) | size[3]; }
};

/**** v2.2 ****/

struct id3v2_2_tag {
	char	identifier[3];
	uint8	size[3];

	uint32 Tag() const { return (identifier[0] << 16) | (identifier[1] << 8) | (identifier[2]); }
	uint32 Size() const { return (size[0] << 16) | (size[1] << 8) | (size[2]); }

	char *DataAt(int index) const { return (char *)(this + 1) + 1 + index; }
	char *Data() const { return DataAt(0); }
	uint8 Charset() const { return *DataAt(-1); }
};

#define	ID2_TRACK			'TRK'
#define ID2_ENCODER			'TEN'
//#define ID2_IDENTIFIER		'TCOP'
//#define ID2_ORIGINAL_ARTIST	'TOPE'
//#define ID2_COMPOSER		'TCOM'
#define ID2_GENRE			'TCO'
#define ID2_COMMENTS		'COM'
#define ID2_YEAR			'TYE'
#define ID2_ALBUM			'TAL'
#define ID2_ARTIST			'TP1'
#define ID2_TITLE			'TT2'

/**** v2.3 ****/

struct id3v2_3_tag {
	uint32	identifier;
	uint32	size;
	uint16	flags;

	uint32 Tag() const { return B_BENDIAN_TO_HOST_INT32(identifier); }
	uint32 Size() const { return B_BENDIAN_TO_HOST_INT32(size); }

	char *DataAt(int index) const { return (char *)(this + 1) + 1 + index; }
	char *Data() const { return DataAt(0); }
	uint8 Charset() const { return *DataAt(-1); }
};

#define	ID3_TRACK			'TRCK'
#define ID3_ENCODER			'TENC'
#define ID3_IDENTIFIER		'TCOP'
#define ID3_ORIGINAL_ARTIST	'TOPE'
#define ID3_COMPOSER		'TCOM'
#define ID3_GENRE			'TCON'
#define ID3_COMMENTS		'COMM'
#define ID3_YEAR			'TYER'
#define ID3_ALBUM			'TALB'
#define ID3_ARTIST			'TPE1'
#define ID3_TITLE			'TIT2'


/************************* Audio attributes to store *********************/

struct audio_attrs {
	char *title;
	char *artist;
	char *album;
	char *comment;
	int32 track;
	int32 year;
	char *genre;
	int32 bitrate;
	bigtime_t length;
	float framerate;
};

#endif  /* ID3ATTR_H */
